; Copyright (C) 2001 Ahmed Abdel-Hamid Mohamed <ahmedam@mail.usa.com>

	org 100h
	section .text
	
	;initialize segments
	mov ax,cs
	mov ds,ax
	mov es,ax
	cli
	mov ss,ax
	mov sp,stackend
	sti
	
	;welcome
	mov ah,9
	mov dx,welcome_msg
	int 21h

	;the buffer shouldn't cross 64k boundary
	mov word [bufp],buf1
	mov bx,buf1
	shl ax,4
	add ax,bx
	add ax,1024
	jno j1
	mov word [bufp],buf2		;buf1 is across boundary
j1:

	;try to open the file, if it is found read the sector from it
	;and restore the MBR
	mov ah,3dh
	mov al,0	;read only
	mov dx,fname
	int 21h
	jc savembr	;file not found, try to create it
	mov [fd],ax

	;read first the MBR to get the partition table
	call readmbr

	;read the file and restore MBR
	mov bx,[fd]	;file handle
	mov cx,446		;ignore partition table
	mov dx,[bufp]
	mov ah,3fh
	int 21h
	jnc done1
	;error reading file
	mov ah,9
	mov dx,readf_err_msg
	int 21h
	jmp exit
done1:
	;close the file
	mov bx,[fd]
	mov ah,3eh
	int 21h
	
	;verify first
	mov ah,9
	mov dx,verify_write_msg
	int 21h
	;get answer from user
	mov ah,1
	int 21h
	push ax
	;new line
	mov ah,9
	mov dx,new_line
	int 21h
	pop ax
	or al,20h	;tolower()
	cmp al,'y'
	jne exit
	
	;restore MBR
	call writembr
	
	;done msg
	mov ah,9
	mov dx,done_w_msg
	int 21h
	
exit:
	;exit program
	mov ah,4ch
	mov al,0
	int 21h

;save the MBR sector into file mbr.b
savembr:
	call readmbr
	;mbr saved in buf
	
	;create the file
	mov ah,3ch
	mov cx,0	;read only
	mov dx,fname
	int 21h
	jc errorc	;file can't be created
	
	;save buf contents
	mov [fd],ax
	mov bx,ax	;file handle
	mov cx,512
	mov dx,[bufp]
	mov ah,40h
	int 21h
	jc errorc
	
	;close file
	mov bx,[fd]
	mov ah,3eh
	int 21h

	;done msg
	mov ah,9
	mov dx,done_r_msg
	int 21h
	
	;exit program
	jmp exit
	
errorc:
	mov ah,9
	mov dx,writef_err_msg
	int 21h
	;exit program
	jmp exit
			
	times 4000 db 0	;stack
stackend dd 0
fd	dw 0			;file handle
buf1	times 1028 db 0	;sector buffer
buf2	times 1028 db 0	;sector buffer
bufp	dw buf1
welcome_msg	db "MBR restoring utility. By Ahmed Abdel-Hamid (ahmedam@mail.usa.com)",0dh,0ah,"$"
read_err_msg	db "Error reading MBR",0dh,0ah,"$"
readf_err_msg	db "Error reading MBR.b file",0dh,0ah,"$"
write_err_msg	db "Error writing MBR",0dh,0ah,"$"
writef_err_msg	db "Error writing MBR.b file",0dh,0ah,"$"
new_line	db 0dh,0ah,"$"
verify_write_msg	db "I'll restore the MBR sector, is this what you want (y/n)? ","$"
done_w_msg	db "MBR restored",0dh,0ah,"$"
done_r_msg	db "MBR saved",0dh,0ah,"$"
fname	db "mbr.b",0
_i	dw 5			;used for retry

;read the buffer
readmbr:
	mov word [_i],5
readagain:
	mov dl,80h		;drive 0
	mov bx,[bufp]	;es:bx=buf
	mov cx,1		;sector | track<<4
	mov dh,0		;head number
	mov ah,2h
	mov al,1		;number of sectors
	int 13h
	jb read_err
	ret
read_err:
	dec word [_i]
	jnz readagain
	mov ah,9
	mov dx,read_err_msg
	int 21h
	jmp exit

writembr:
;write the buffer
	mov word [_i],5
writeagain:
	mov dl,80h		;drive 0
	mov bx,[bufp]	;es:bx=buf1
	mov cx,1		;sector | track<<4
	mov dh,0		;head number
	mov ah,3h
	mov al,1		;number of sectors
	int 13h
	jb write_err
	
	ret
write_err:
	dec word [_i]
	jnz writeagain
	mov ah,9
	mov dx,write_err_msg
	int 21h
	jmp exit
